//------------------------------------------------------------------
#property copyright "mladen"
#property link      "mladenfx@gmail.com"
//------------------------------------------------------------------
#property indicator_separate_window
#property indicator_buffers 5
#property indicator_color1  LimeGreen
#property indicator_color2  Magenta
#property indicator_color3  RoyalBlue
#property indicator_color4  Red
#property indicator_color5  DarkSlateGray
#property indicator_style2  STYLE_DOT
#property indicator_width3  2
#property indicator_width4  2
#property indicator_width5  2


//
//
//
//
//

extern string TimeFrame       = "Current time frame";
extern int    TSIPeriod1      = 25;
extern int    TSIPeriod2      = 13;
extern int    SignalPeriod    = 7;
extern int    Price           = PRICE_CLOSE;
extern int    histoMultiplier = 1;

double TSIBuffer[];
double SignalBuffer[];
double oscUp[];
double oscDn[];
double osc[];
double trend[];

string indicatorFileName;
bool   returnBars;
bool   calculateValue;
int    timeFrame;

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int init()
{
   IndicatorBuffers(6);
   SetIndexBuffer(0,TSIBuffer);    SetIndexLabel(0,"TSI");
   SetIndexBuffer(1,SignalBuffer); SetIndexLabel(1,"TSI Signal");
   SetIndexBuffer(2,oscUp);        SetIndexStyle(2,DRAW_HISTOGRAM);
   SetIndexBuffer(3,oscDn);        SetIndexStyle(3,DRAW_HISTOGRAM);
   SetIndexBuffer(4,osc);
   SetIndexBuffer(5,trend);
   
      //
      //
      //
      //
      //
      
         indicatorFileName = WindowExpertName();
         calculateValue    = TimeFrame=="calculateValue"; if (calculateValue) { return(0); }
         returnBars        = TimeFrame=="returnBars";     if (returnBars)     { return(0); }
         timeFrame         = stringToTimeFrame(TimeFrame);
      
      //
      //
      //
      //
      //

   IndicatorShortName(timeFrameToString(timeFrame)+" TSI ("+TSIPeriod1+","+TSIPeriod2+","+SignalPeriod+")");
   return(0);
}

//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { TSIBuffer[0] = MathMin(limit+1,Bars-1); return(0); }

   //
   //
   //
   //
   //
   
   if (calculateValue || timeFrame == Period())
   {
      for (int i=limit; i>=0; i--)
      {
         double priceDiff = iMA(NULL,0,1,0,MODE_SMA,Price,i)-iMA(NULL,0,1,0,MODE_SMA,Price,i+1);
         double avg = iEma(iEma(        priceDiff ,TSIPeriod1,i,0),TSIPeriod2,i,1);
         double ava = iEma(iEma(MathAbs(priceDiff),TSIPeriod1,i,2),TSIPeriod2,i,3);
            if (ava != 0)
                  TSIBuffer[i] = 100.0*avg/ava;
            else  TSIBuffer[i] = 0.00;
            SignalBuffer[i] = iEma(TSIBuffer[i],SignalPeriod,i,4);
            osc[i] = (TSIBuffer[i]-SignalBuffer[i])-histoMultiplier;
            
            
            //
            //
            //
            //
            //
            
            oscUp[i] = EMPTY_VALUE; 
            oscDn[i] = EMPTY_VALUE;
            trend[i] = trend[i+1];
            if (osc[i]>0) trend[i] = 1;
            if (osc[i]<0) trend[i] =-1;
            if (trend[i] == 1) oscUp[i] = osc[i];
            if (trend[i] ==-1) oscDn[i] = osc[i];
      }
      return(0);
   }
   
   //
   //
   //
   //
   //
   
   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for (i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         TSIBuffer[i]    = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",TSIPeriod1,TSIPeriod2,SignalPeriod,Price,histoMultiplier,0,y);
         SignalBuffer[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",TSIPeriod1,TSIPeriod2,SignalPeriod,Price,histoMultiplier,1,y);
         oscUp[i]        = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",TSIPeriod1,TSIPeriod2,SignalPeriod,Price,histoMultiplier,2,y);
         oscDn[i]        = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",TSIPeriod1,TSIPeriod2,SignalPeriod,Price,histoMultiplier,3,y);
         osc[i]          = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",TSIPeriod1,TSIPeriod2,SignalPeriod,Price,histoMultiplier,4,y);
         trend[i]        = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",TSIPeriod1,TSIPeriod2,SignalPeriod,Price,histoMultiplier,5,y);
   }
   return(0);         
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

double workEma[][5];
double iEma(double price, double period, int r, int instanceNo=0)
{
   if (ArrayRange(workEma,0)!= Bars) ArrayResize(workEma,Bars); r = Bars-r-1;

   //
   //
   //
   //
   //
      
   double alpha = 2.0 / (1.0+period);
          workEma[r][instanceNo] = workEma[r-1][instanceNo]+alpha*(price-workEma[r-1][instanceNo]);
   return(workEma[r][instanceNo]);
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}